@@ -5,8 +5,9 @@
 ;; Author: Kostafey <kostafey@gmail.com>
 ;; URL: https://github.com/kostafey/ejc-sql
 ;; Keywords: sql, jdbc
-;; Version: 0.4.0
-;; Package-Requires: ((emacs "26.3")(clomacs "0.0.4")(dash "2.16.0")(spinner "1.7.3")(direx "1.0.0"))
+;; Package-Version: 20241111.117
+;; Package-Revision: 1fc5a38d974a
+;; Package-Requires: ((emacs "26.3")(clomacs "0.0.5")(dash "2.16.0")(spinner "1.7.3"))
 
 ;; This file is not part of GNU Emacs.
 
@@ -38,7 +39,6 @@
 (require 'cl-lib)
 (require 'org-table)
 (require 'ejc-lib)
-(require 'ejc-direx)
 (require 'ejc-eldoc)
 (require 'ejc-format)
 (require 'ejc-interaction)
@@ -111,6 +111,14 @@ results. When nil, otherwise, provide `ejc-sql' users expected behaviour."
   :group 'ejc-sql
   :type '(plist :key-type string :value-type (vector symbol string)))
 
+(defcustom ejc-completion-system 'standard
+  "The completion system used by `ejc-connect'."
+  :group 'ejc-sql
+  :type '(radio
+          (const :tag "Ido" ido)
+          (const :tag "Standard" standard)
+          (function :tag "Custom function")))
+
 (defvar-local ejc-replace-double-quotes nil
   "When t replace double quotes with single ones in SQL before evaluation.")
 
@@ -126,6 +134,7 @@ results. When nil, otherwise, provide `ejc-sql' users expected behaviour."
 (define-key ejc-sql-mode-keymap (kbd "C-g") 'ejc-cancel-query)
 (define-key ejc-sql-mode-keymap (kbd "M-.") 'ejc-describe-entity)
 (define-key ejc-sql-mode-keymap (kbd "M-,") 'ejc-show-prev-result)
+(define-key ejc-sql-mode-keymap (kbd ".") 'ejc-dot-pressed)
 
 (defvar ejc-command-map
   (let ((map (make-sparse-keymap)))
@@ -276,6 +285,7 @@ results. When nil, otherwise, provide `ejc-sql' users expected behaviour."
                                  dependencies
                                  classpath
                                  separator
+                                 sslmode
                                  ;; ----------
                                  ;; Optional:
                                  classname)
@@ -313,10 +323,28 @@ For more details about parameters see `get-connection' function in jdbc.clj:
                                            (-map 'file-truename classpath))
                                   (vector (file-truename classpath)))))
                         (cons :separator separator)
+                        (cons :sslmode sslmode)
                         (cons :classname classname)))
                  new-connection))
               ejc-connections)))
 
+(cl-defun ejc-completing-read (prompt choices &key initial-input)
+  "Present a PROMPT with CHOICES and optional INITIAL-INPUT."
+  (cond
+   ((eq ejc-completion-system 'ido)
+    (ido-completing-read prompt choices nil nil initial-input))
+   ((eq ejc-completion-system 'standard)
+    (completing-read prompt choices nil nil initial-input))
+   (t (funcall ejc-completion-system prompt choices))))
+
+(defun ejc-read-file-name (prompt)
+  "Read file name, prompting with PROMPT."
+  (cond
+   ((eq ejc-completion-system 'ido)
+    (ido-read-file-name prompt))
+   (t
+    (read-file-name prompt))))
+
 (defun ejc-find-connection (connection-name)
   "Return pair with name CONNECTION-NAME and db connection structure from
 `ejc-connections'."
@@ -342,7 +370,11 @@ For more details about parameters see `get-connection' function in jdbc.clj:
 (defun ejc-set-mode-name (connection-name)
   "Show CONNECTION-NAME as part of `mode-name' in `mode-line'."
   (setq mode-name (format "%s->[%s]"
-                          (car (split-string mode-name "->\\[.+\\]"))
+                          (car (split-string
+                                (if (listp mode-name)
+                                    (car mode-name)
+                                  mode-name)
+                                "->\\[.+\\]"))
                           connection-name)))
 
 (cl-defun ejc-add-connection (&optional connection-name db)
@@ -366,18 +398,19 @@ If the current mode is `sql-mode' prepare buffer to operate as `ejc-sql-mode'."
                  (concat "ejc-sql is enabled, ignore source block connection"
                          " header arguments and use ejc-sql to execute it? ")))))
       (funcall orig-fun body params)
-    (cl-multiple-value-bind (beg end) (save-mark-and-excursion
-                                        (org-babel-mark-block)
-                                        (list (point) (mark)))
-      (ejc-eval-user-sql-at-point
-       :beg beg
-       :end end
+    (let* ((info (org-babel-get-src-block-info 'no-eval))
+           (expanded-body (if (org-babel-noweb-p (nth 2 info) :eval)
+                              (org-babel-expand-noweb-references info)
+                            (nth 1 info))))
+      (ejc-eval-user-sql
+       expanded-body
        :sync ejc-org-mode-show-results
        :display-result (not ejc-org-mode-show-results))
       (if ejc-org-mode-show-results
           (with-temp-buffer
             (insert-file-contents (ejc-get-result-file-path))
-            (buffer-string))))))
+            (or (org-babel-read-table)
+                (buffer-string)))))))
 
 (defun ejc-org-edit-special (orig-fun &rest args)
   (if (and (equal "sql" (car (org-babel-get-src-block-info)))
@@ -391,7 +424,7 @@ If the current mode is `sql-mode' prepare buffer to operate as `ejc-sql-mode'."
 
 (defun ejc-read-connection-name ()
   "Read connection-name in minibuffer."
-  (ido-completing-read
+  (ejc-completing-read
    "DataBase connection name: "
    (let ((conn-list (mapcar 'car ejc-connections))
          (conn-statistics (ejc-load-conn-statistics)))
@@ -456,7 +489,7 @@ Apropriate artifacts list located in `ejc-jdbc-drivers'."
                      ("SQLite" . "sqlite")
                      ("PostgreSQL" . "postgresql")))
          (dbtype (cdr (assoc-string
-                       (ido-completing-read
+                       (ejc-completing-read
                         "Database type: "
                         (cl-sort (-map 'car db-types)
                                  'string-lessp :key 'downcase))
@@ -509,7 +542,7 @@ Apropriate artifacts list located in `ejc-jdbc-drivers'."
                  (:subname (if (member dbtype '("h2" "sqlite"))
                                (let ((fpath
                                       (file-truename
-                                       (ido-read-file-name "DB file: "))))
+                                       (ejc-read-file-name "DB file: "))))
                                  (if (equal "h2" dbtype)
                                      (concat "file://"
                                              (file-name-sans-extension
@@ -601,33 +634,6 @@ any SQL buffer to connect to exact database, as always. "
     (orgtbl-mode     t)
     (ejc-result-mode nil)))
 
-(cl-defun ejc-eval-sql-and-log (db
-                                sql
-                                &key
-                                start-time
-                                rows-limit
-                                column-width-limit
-                                append
-                                sync
-                                display-result
-                                result-file)
-  (when sql
-    (spinner-start 'rotating-line)
-    (setq ejc-current-buffer-query (current-buffer))
-    (let* ((prepared-sql (ejc-get-sql-from-string sql)))
-      (ejc--eval-sql-and-log-print
-       db
-       prepared-sql
-       :start-time start-time
-       :rows-limit rows-limit
-       :column-width-limit column-width-limit
-       :append append
-       :sync sync
-       :display-result display-result
-       :result-file (or result-file
-                        (ejc-next-result-file-path))
-       :add-outside-borders (ejc-add-outside-borders-p)))))
-
 (defun ejc-message-query-done (start-time status)
   (message
    "%s SQL query at %s. Exec time %.03f"
@@ -728,19 +734,33 @@ brackets is a symbol under point (cursor). Return a list of two items:
                               :result-file (ejc-next-result-file-path)))
 
 (cl-defun ejc-eval-user-sql (sql &key
+                                 db
                                  rows-limit
+                                 fetch-size
                                  column-width-limit
+                                 append
                                  sync
-                                 display-result)
+                                 display-result
+                                 result-file)
   "User starts SQL evaluation process."
   (message "Processing SQL query...")
-  (ejc-eval-sql-and-log  ejc-db
-                         sql
-                         :rows-limit rows-limit
-                         :column-width-limit column-width-limit
-                         :start-time (current-time)
-                         :sync sync
-                         :display-result display-result))
+  (when sql
+    (spinner-start 'rotating-line)
+    (setq ejc-current-buffer-query (current-buffer))
+    (let* ((prepared-sql (ejc-get-sql-from-string sql)))
+      (ejc--eval-sql-and-log-print
+       (or db ejc-db)
+       prepared-sql
+       :start-time (current-time)
+       :rows-limit rows-limit
+       :fetch-size fetch-size
+       :column-width-limit column-width-limit
+       :append append
+       :sync sync
+       :display-result display-result
+       :result-file (or result-file
+                        (ejc-next-result-file-path))
+       :add-outside-borders (ejc-add-outside-borders-p)))))
 
 ;;;###autoload
 (defun ejc-eval-user-sql-region (beg end)
@@ -777,6 +797,7 @@ boundaries."
   (ejc-eval-user-sql
    (ejc-select-db-meta-script ejc-db :all-tables)
    :rows-limit 0
+   :fetch-size 0
    :column-width-limit 0
    :display-result t))
 
@@ -845,7 +866,6 @@ Buffer can be saved to file with `ejc-temp-editor-file' path."
         (find-file tmp-file-path)
         (rename-buffer tmp-buffer-name)
         (sql-mode)
-        (auto-fill-mode t)
         (ejc-add-connection)
         (get-buffer tmp-buffer-name)))))
 
